--[[
	BOHooks
	 	Contains the hooked functions for BarOptions
	
	By: Mugendai
	Contact: mugekun@gmail.com
	
	$Id: BOHooks.lua 4715 2007-02-15 00:05:43Z karlkfi $
	$Rev: 4715 $
	$LastChangedBy: karlkfi $
	$Date: 2007-02-14 18:05:43 -0600 (Wed, 14 Feb 2007) $
]]--

--------------------------------------------------
--
-- Global Variables
--
--------------------------------------------------
--used to keep up with the last page the bar was on
Instant_ActionBar_Return = -1;

--------------------------------------------------
--
-- Hook Function
--
--------------------------------------------------
BarOptions.Hook = function ()
	--ID action bar change hook functions
	hooksecurefunc("ActionBar_PageUp", BarOptions.ActionBar_PageUp);
	hooksecurefunc("ActionBar_PageDown", BarOptions.ActionBar_PageDown);
	hooksecurefunc("ChangeActionBarPage", BarOptions.ChangeActionBarPage);

	--Empty button and button count hook functions
	hooksecurefunc("ActionButton_Update", BarOptions.ActionButton_Update);
	hooksecurefunc("ActionButton_ShowGrid", BarOptions.ShowButton);
	hooksecurefunc("ActionButton_HideGrid", BarOptions.ShowButton);
	
	--Hotkey mods hook functions
	MCom.util.hook("ActionButton_UpdateHotkeys", "BarOptions.ActionButton_UpdateHotkeys", "replace");
	
	--Instant Action Bar hook functions
	hooksecurefunc("ActionButtonDown", BarOptions.ActionButtonDown);
	hooksecurefunc("ActionButtonUp", BarOptions.ActionButtonUp);
	
	--Range Coloring hook functions
	hooksecurefunc("ActionButton_UpdateUsable", BarOptions.ActionButton_UpdateUsable);
	hooksecurefunc("ActionButton_OnUpdate", BarOptions.ActionButton_OnUpdate);
                                                                                                                                                                          
	--Hook MobileFrames reset functions to update bar position
	if (MobileFrames_SpecialFrames and MobileFrames_SpecialFrames.MainMenuBar and MobileFrames_SpecialFrames.MainMenuBar.reset) then
		MCom.util.hook("MobileFrames_SpecialFrames.MainMenuBar.reset", "BarOptions.MoveMainBar", "after");
	end
end;

--------------------------------------------------
--
-- Hooks, ActionButton
--
--------------------------------------------------
function BarOptions.ActionButton_UpdateHotkeys(actionButtonType, button)
	if (not button) then
		button = this;
	end
	local hotkey = getglobal(button:GetName().."HotKey");
	local pagedButtonID = ActionButton_GetPagedID(button);
	local buttonID = button:GetID();
	if(buttonID < 1) then
		buttonID = (SecureButton_GetModifiedAttribute(button, "action", SecureStateChild_GetEffectiveButton(button)) or 1);
		if(buttonID > 12) then
			buttonID = math.fmod(buttonID, 12);
			if(buttonID == 0) then
				buttonID = 12;
			end
		end
	end
	
	if ( not actionButtonType ) then
		--Set the actionButtonType based on which bar this button is associated with
		if (string.find(button:GetName(), "MultiBarBottomLeft")) then
			actionButtonType = "MULTIACTIONBAR1BUTTON";
		elseif (string.find(button:GetName(), "MultiBarBottomRight")) then
			actionButtonType = "MULTIACTIONBAR2BUTTON";
		elseif (string.find(button:GetName(), "MultiBarRight")) then
			actionButtonType = "MULTIACTIONBAR3BUTTON";
		elseif (string.find(button:GetName(), "MultiBarLeft")) then
			actionButtonType = "MULTIACTIONBAR4BUTTON";
		else
			actionButtonType = "ACTIONBUTTON";
		end
	end
	local action = actionButtonType..buttonID;
	local key = GetBindingKey(action) or GetBindingKey("CLICK "..button:GetName()..":LeftButton");
	
	local shortkeys = false;
	if(BarOptions_Config.ShortKeys == 1) then
		shortkeys = true;
	end
	
	if (hotkey) then
		local keyText = "";
		--If hiding key bindings is disabled and we have a binding, show it
		if (BarOptions_Config.HideKeys==0 and action) then
			keyText = GetBindingText(key, "KEY_", shortkeys);
			if(key and (keyText == "·")) then
				keyText = key;
				if(keyText and shortkeys) then
					for str, repl in pairs(BO_SHORTHOTKEYS) do
						keyText = string.gsub(keyText, str, repl);
					end
				end
			end
			if (BarOptions_Config.HideKeyMod==1) then
				--If we are supposed to hide the key modifier, then do so
				while ( ( keyText ) and ( strlen(keyText) > 1 ) and (string.find(keyText, "-")) and (keyText ~= "Num Pad -") and (keyText ~= "-")) do
					firsti, lasti, keyText = string.find(keyText, "-(.+)");
				end
			end
			if ( ( not keyText ) or ( strlen(keyText) <= 0 ) ) then keyText = ""; end
			if(keyText == "") then
				if(HasAction(pagedButtonID) and ActionHasRange(pagedButtonID) and IsActionInRange(pagedButtonID)) then
					keyText = RANGE_INDICATOR;
					--hotkey:SetTextHeight(8);
					--hotkey:SetPoint("TOPRIGHT", button:GetName(), "TOPRIGHT", -3, 5);
				end
			end
		end
		hotkey:SetText(keyText);
	end
end

local BOActionButtonPushedIDs = {};
function BarOptions.ActionButtonDown(id)
	local button;
	if ( BonusActionBarFrame:IsShown() ) then
		button = getglobal("BonusActionButton"..id);
	else
		button = getglobal("ActionButton"..id);
	end
	if ( button:GetButtonState() == "PUSHED" ) then
		BOActionButtonPushedIDs[id] = true;
	end
end

function BarOptions.ActionButtonUp(id)
	local button;
	if ( BonusActionBarFrame:IsVisible() ) then
		button = getglobal("BonusActionButton"..id);
	else
		local button = getglobal("ActionButton"..id);
	end
	if ( BOActionButtonPushedIDs[id] and Instant_ActionBar_Return ~= -1 ) then
		for k,_ in pairs(BOActionButtonPushedIDs) do
			BOActionButtonPushedIDs[k] = nil;
		end
--	Instant Action Bar START
		ChangeActionBarPage(Instant_ActionBar_Return);
		Instant_ActionBar_Return = -1;
--	Instant Action Bar end
	end
end

function BarOptions.ActionButton_UpdateUsable()
	local icon = getglobal(this:GetName().."Icon");
	local normalTexture = getglobal(this:GetName().."NormalTexture");
	local isUsable, notEnoughMana = IsUsableAction(ActionButton_GetPagedID(this));
	if ( isUsable and (BarOptions_Config.RangeColorOn == 1) and this.rangeTimer and (IsActionInRange(ActionButton_GetPagedID(this)) == 0)) then
		icon:SetVertexColor(BarOptions_Config.RangeColor.r, BarOptions_Config.RangeColor.g, BarOptions_Config.RangeColor.b);
		normalTexture:SetVertexColor(BarOptions_Config.RangeColor.r, BarOptions_Config.RangeColor.g, BarOptions_Config.RangeColor.b);
	end
end

function BarOptions.ActionButton_OnUpdate(elapsed)
	if(this.rangeTimer and (math.abs(this.rangeTimer - TOOLTIP_UPDATE_TIME) < 0.000001)) then
		if(BarOptions_Config.RangeColorOn == 1) then
			ActionButton_UpdateUsable();
		end
	end
end

function BarOptions.ActionButton_Update()
	if ( this.isBonus and this.inTransition ) then
		return;
	end
	
	BarOptions.ShowButton(this);
end

function BarOptions.ActionBar_PageUp()
	local currentPage = GetActionBarPage();
	local nextPage = currentPage;
	
	if(BarOptions_Config.AlternateIDs == 1) then
		if(BarOptions_Config.GroupPages == 1) then
			--If grouping is enabled, then simply don't exceed 3 pages
			if((currentPage+1) <= (NUM_ACTIONBAR_PAGES / 2)) then
				nextPage = (currentPage+1);
			end
--		else
--			--If alternate is enabled, then simply don't exceed 6 pages
--			if ((currentPage+1) <= NUM_ACTIONBAR_PAGES) then
--				nextPage = (currentPage+1);
--			end
		end
	end
	
	if(nextPage ~= currentPage) then
		ChangeActionBarPage(nextPage);
	end
end

function BarOptions.ActionBar_PageDown()
	local currentPage = GetActionBarPage();
	local prevPage = currentPage;
	
	if((BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.GroupPages == 1)) then
		--If we are grouping flip to page 3, if we go below 1
		prevPage = (NUM_ACTIONBAR_PAGES / 2);
	end
	
	if((BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.GroupPages == 1)) then
		--Simply don't go below page 1
		if((currentPage-1) > 0) then
			prevPage = (currentPage-1);
		end
	end
	
	if(prevPage ~= currentPage) then
		ChangeActionBarPage(prevPage);
	end
end

local BO_Changing_Page = false;
function BarOptions.ChangeActionBarPage(page)
	if((not BO_Changing_Page) and (BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.TurnPages == 1) and (BarOptions_Config.GroupPages == 1) and (page > (NUM_ACTIONBAR_PAGES / 2))) then
		BO_Changing_Page = true;
--		if(this) then
--			if(this:GetName() == "ActionBarDownButton") then
--				ChangeActionBarPage(1);
--			elseif(this:GetName() == "ActionBarUpButton") then
--				ChangeActionBarPage((NUM_ACTIONBAR_PAGES / 2));
--			end
--		else
			ChangeActionBarPage(page - (NUM_ACTIONBAR_PAGES / 2));
--		end
		BO_Changing_Page = false;
	end
end
