--------------------------------------------------------------------------
-- UIPanelOptions.lua 
--------------------------------------------------------------------------
--[[
UIPanel Options

By: AnduinLothar    <KarlKFI@cosmosui.org>
		
	$Id: UIPanelOptions.lua 4821 2007-03-19 08:12:06Z karlkfi $
	$Rev: 4821 $
	$LastChangedBy: karlkfi $
	$Date: 2007-03-19 08:12:06Z $


]]--

-- Only work for wow 2.1 or greater
local wowvers = select(3, strfind(GetBuildInfo(), "%d+%.(%d+)%."))
if (wowvers and tonumber(wowvers) < 1) then
	return
end

UIPanelOptions = {}

-- <= == == == == == == == == == == == == =>
-- => GroupLootFrame Stacking
-- <= == == == == == == == == == == == == =>

function UIPanelOptions.ToggleRollStack()

	if (MobileFrames_RollStackDown) then
		MobileFrames_RollStackDown = nil
		UIPanelOptions.SetDownwardRollWindowStack(0)
	else
		MobileFrames_RollStackDown = true
		UIPanelOptions.SetDownwardRollWindowStack(1)
	end

end

function UIPanelOptions.SetDownwardRollWindowStack(toggle)
	if (toggle == 1) then
		GroupLootFrame2:ClearAllPoints()
		GroupLootFrame2:SetPoint("TOP", "GroupLootFrame1", "BOTTOM", 0, -3)
		GroupLootFrame3:ClearAllPoints()
		GroupLootFrame3:SetPoint("TOP", "GroupLootFrame2", "BOTTOM", 0, -3)
		GroupLootFrame4:ClearAllPoints()
		GroupLootFrame4:SetPoint("TOP", "GroupLootFrame3", "BOTTOM", 0, -3)
	else
		GroupLootFrame2:ClearAllPoints()
		GroupLootFrame2:SetPoint("BOTTOM", "GroupLootFrame1", "TOP", 0, 3)
		GroupLootFrame3:ClearAllPoints()
		GroupLootFrame3:SetPoint("BOTTOM", "GroupLootFrame2", "TOP", 0, 3)
		GroupLootFrame4:ClearAllPoints()
		GroupLootFrame4:SetPoint("BOTTOM", "GroupLootFrame3", "TOP", 0, 3)
	end
end

-- <= == == == == == == == == == == == == =>
-- => Offset Marker
-- <= == == == == == == == == == == == == =>

function UIPanelOptions.Marker_ShowLeft(offset)
	if (type(offset) == "number") then
		local marker = UIPanelHOffsetsMarker
		marker:ClearAllPoints()
		marker:SetPoint("CENTER", "UIParent", "LEFT", offset, 0)
		marker:Show()
		UIPanelOptions.Marker_HFade()
	end
end

function UIPanelOptions.Marker_ShowRight(offset)
	if (type(offset) == "number") then
		local marker = UIPanelHOffsetsMarker
		marker:ClearAllPoints()
		marker:SetPoint("CENTER", "UIParent", "RIGHT", offset, 0)
		marker:Show()
		UIPanelOptions.Marker_HFade()
	end
end

function UIPanelOptions.Marker_ShowTop(offset)
	if (type(offset) == "number") then
		local marker = UIPanelVOffsetsMarker
		marker:ClearAllPoints()
		marker:SetPoint("CENTER", "UIParent", "TOP", 0, offset)
		marker:Show()
		UIPanelOptions.Marker_VFade()
	end
end

function UIPanelOptions.Marker_ShowBottom(offset)
	-- Offset currently specific for bags
	if (type(offset) == "number") then
		local marker = UIPanelVOffsetsMarker
		marker:ClearAllPoints()
		marker:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", 40, offset-16)
		marker:Show()
		UIPanelOptions.Marker_VFade()
	end
end

function UIPanelOptions.Marker_HFade()
	local marker = UIPanelHOffsetsMarker
	if (marker:IsVisible()) then
		marker:SetAlpha(1)
		marker.counter = 80 -- ~ 2 sec timer
		marker.fadeCount = 40  -- fade when ~ 1 sec left
		marker.isFading = true
	end
end

function UIPanelOptions.Marker_VFade()
	local marker = UIPanelVOffsetsMarker
	if (marker:IsVisible()) then
		marker:SetAlpha(1)
		marker.counter = 80 -- ~ 2 sec timer
		marker.fadeCount = 40  -- fade when ~ 1 sec left
		marker.isFading = true
	end
end

function UIPanelOptions.Marker_HHide()
	local marker = UIPanelHOffsetsMarker
	marker.isFading = false
	marker.counter = nil
	marker.fadeCount = nil
	marker:Hide()
end

function UIPanelOptions.Marker_VHide()
	local marker = UIPanelVOffsetsMarker
	marker.isFading = false
	marker.counter = nil
	marker.fadeCount = nil
	marker:Hide()
end

function UIPanelOptions.Marker_OnUpdate(self)
	if (not self) then
		self = this
	end
	if (self.isFading) then
		if (type(self.counter) == "number") then
			if (self.counter > 0) then
				self.counter = self.counter - 1
				if (type(self.fadeCount) == "number") then
					if (self.counter < self.fadeCount) then
						self:SetAlpha(self.counter/self.fadeCount)
					end
				end
			else
				self.counter = nil
				self.fadeCount = nil
			end
		else
			self.isFading = false
			self:Hide()
			self:SetAlpha(1)
		end
	end
end


-- <= == == == == == == == == == == == == =>
-- => UIPanel Offset
-- <= == == == == == == == == == == == == =>

function UIPanelOptions.SetLeftPanelOffset(checked, value)
	if (type(value) == "number") and (checked == 1) then
		if (value >= 0) then
			UIParent:SetAttribute("LEFT_OFFSET", value)
			if (not Khaos) then
				UIPanelOptions.Marker_ShowLeft(value)
			end
		end
	end
end

function UIPanelOptions.SetRightPanelOffset(checked, value)
	if (type(value) == "number") and (checked == 1) then
		if (value <= 0) then
			UIParent:SetAttribute("RIGHT_OFFSET_BUFFER", -value)
			if (not Khaos) then
				UIPanelOptions.Marker_ShowRight(value)
			end
		end
	end
end

function UIPanelOptions.SetTopPanelOffset(checked, value)
	if (type(value) == "number") and (checked == 1) then
		if (value <= 0) then
			UIParent:SetAttribute("TOP_OFFSET", value)
			if (not Khaos) then
				UIPanelOptions.Marker_ShowTop(value)
			end
		end
	end
end


-- <= == == == == == == == == == == == == =>
-- => Bags Offset
-- <= == == == == == == == == == == == == =>

function UIPanelOptions.GetRightBagsOffset()
	return UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_X.baseX
end

function UIPanelOptions.SetRightBagsOffset(checked, value)
	if (type(value) == "number") and (checked == 1) then
		if (value <= 0) then
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_X.baseX = -value
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_X.rightLeft = 0
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_X.rightRight = 0
			if (not Khaos) then
				UIPanelOptions.Marker_ShowRight(value)
			end
			UIParent_ManageFramePositions()
		end
	end
end

function UIPanelOptions.SetBottomBagsOffset(checked, value)
	if (type(value) == "number") and (checked == 1) then
		if (value >= 0) then
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_Y.baseY = value
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_Y.bottomEither = 0
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_Y.reputation = 0
			UIPARENT_MANAGED_FRAME_POSITIONS.CONTAINER_OFFSET_Y.pet = 0
			if (not Khaos) then
				UIPanelOptions.Marker_ShowBottom(value)
			end
			UIParent_ManageFramePositions()
		end
	end
end


-- <= == == == == == == == == == == == == =>
-- => Khaos Regs
-- <= == == == == == == == == == == == == =>

function UIPanelOptions.RegisterForKhaos()
	local optionSet = {
		id="UIPanelOptions",
		text="UIPanel Options",
		helptext="Khaos sliders for setting the offset of UIPanels, bags and border frames",
		default=true,
		options={
			{
				id="Header",
				text="UIPanel Offsets",
				helptext="Khaos sliders for setting the offset of UIPanels, bags and border frames",
				type=K_HEADER,
			},
			{
				id="DownwardsRollWindowStacking",
				type=K_TEXT,
				text=UIPANEL_OPTIONS_DOWN_ROLLSTACK,
				helptext=UIPANEL_OPTIONS_DOWN_ROLLSTACK_INFO,
				callback=function(state)
					UIPanelOptions.SetDownwardRollWindowStack(state.checked and 1 or 0)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						return UIPANEL_OPTIONS_ROLLSTACK_FEEDBACK_ENABLED
					else
						return UIPANEL_OPTIONS_ROLLSTACK_FEEDBACK_DISABLED
					end
				end,
				check=true,
				default={checked=false},
				disabled={checked=false},
			},
			{
				id="SetLeftPanelOffset",
				type=K_SLIDER,
				text=UIPANEL_OPTIONS_SET_PANEL_LEFT_OFFSET,
				helptext=UIPANEL_OPTIONS_SET_PANEL_LEFT_OFFSET_INFO,
				callback=function(state)
					UIPanelOptions.SetLeftPanelOffset(1,state.slider)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						UIPanelOptions.Marker_ShowLeft(state.slider)
						return format(UIPANEL_OPTIONS_PANEL_LEFT_OFFSET_FEEDBACK_ENABLED, state.slider)
					else
						return UIPANEL_OPTIONS_PANEL_LEFT_OFFSET_FEEDBACK_DISABLED
					end
				end,
				check=true,
				default={checked=false,slider=0},
				disabled={checked=false,slider=0},
				dependencies={SetLeftPanelOffset={checked=true,match=true}},
				setup={
					sliderStep=5,
					sliderText=UIPANEL_OFFSET,
					sliderMin=0,
					sliderMax=200,
				},
			},
			{
				id="SetRightPanelOffset",
				type=K_SLIDER,
				text=UIPANEL_OPTIONS_SET_PANEL_RIGHT_OFFSET,
				helptext=UIPANEL_OPTIONS_SET_PANEL_RIGHT_OFFSET_INFO,
				callback=function(state)
					UIPanelOptions.SetRightPanelOffset(1,state.slider)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						UIPanelOptions.Marker_ShowRight(state.slider)
						return format(UIPANEL_OPTIONS_PANEL_RIGHT_OFFSET_FEEDBACK_ENABLED, state.slider)
					else
						return format(UIPANEL_OPTIONS_PANEL_RIGHT_OFFSET_FEEDBACK_DISABLED, state.slider)
					end
				end,
				check=true,
				default={checked=false,slider=-80},
				disabled={checked=false,slider=-80},
				dependencies={SetRightPanelOffset={checked=true,match=true}},
				setup={
					sliderStep=5,
					sliderText=UIPANEL_OFFSET,
					sliderMin=-200,
					sliderMax=0,
				},
			},
			{
				id="SetTopPanelOffset",
				type=K_SLIDER,
				text=UIPANEL_OPTIONS_SET_PANEL_TOP_OFFSET,
				helptext=UIPANEL_OPTIONS_SET_PANEL_TOP_OFFSET_INFO,
				callback=function(state)
					UIPanelOptions.SetTopPanelOffset(1,state.slider)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						UIPanelOptions.Marker_ShowTop(state.slider)
						return format(UIPANEL_OPTIONS_PANEL_TOP_OFFSET_FEEDBACK_ENABLED, state.slider)
					else
						return UIPANEL_OPTIONS_PANEL_TOP_OFFSET_FEEDBACK_DISABLED
					end
				end,
				check=true,
				default={checked=false,slider=-104},
				disabled={checked=false,slider=-104},
				dependencies={SetTopPanelOffset={checked=true,match=true}},
				setup={
					sliderStep=5,
					sliderText=UIPANEL_OFFSET,
					sliderMin=-200,
					sliderMax=0,
				},
			},
			{
				id="SetRightBagsOffset",
				type=K_SLIDER,
				text=UIPANEL_OPTIONS_SET_BAGS_RIGHT_OFFSET,
				helptext=UIPANEL_OPTIONS_SET_BAGS_RIGHT_OFFSET_INFO,
				callback=function(state)
					UIPanelOptions.SetRightBagsOffset((state.checked and 1 or 0),state.slider)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						UIPanelOptions.Marker_ShowRight(state.slider)
						return format(UIPANEL_OPTIONS_BAGS_RIGHT_OFFSET_FEEDBACK_ENABLED, state.slider)
					else
						return UIPANEL_OPTIONS_BAGS_RIGHT_OFFSET_FEEDBACK_DISABLED
					end
				end,
				check=true,
				default={checked=false,slider=0},
				disabled={checked=false,slider=0},
				dependencies={SetRightBagsOffset={checked=true,match=true}},
				setup={
					sliderStep=5,
					sliderText=UIPANEL_OFFSET,
					sliderMin=-200,
					sliderMax=0,
				},
			},
			{
				id="SetBottomBagsOffset",
				type=K_SLIDER,
				text=UIPANEL_OPTIONS_SET_BAGS_BOTTOM_OFFSET,
				helptext=UIPANEL_OPTIONS_SET_BAGS_BOTTOM_OFFSET_INFO,
				callback=function(state)
					UIPanelOptions.SetBottomBagsOffset((state.checked and 1 or 0),state.slider)
				end,
				feedback=function(state)
					if ( state.checked ) then 
						UIPanelOptions.Marker_ShowBottom(state.slider)
						return format(UIPANEL_OPTIONS_BAGS_BOTTOM_OFFSET_FEEDBACK_ENABLED, state.slider)
					else
						return UIPANEL_OPTIONS_BAGS_BOTTOM_OFFSET_FEEDBACK_DISABLED
					end
				end,
				check=true,
				default={checked=false,slider=0},
				disabled={checked=false,slider=0},
				dependencies={SetBottomBagsOffset={checked=true,match=true}},
				setup={
					sliderStep=5,
					sliderText=UIPANEL_OFFSET,
					sliderMin=0,
					sliderMax=200,
				},
			},
		},
	}
	
	Khaos.registerOptionSet("frames", optionSet, true)

end

UIPanelOptions.RegisterForKhaos()
